@@ -1,9 +1,8 @@
 
 from pyjjasim import *
 
-import matplotlib
 import matplotlib.pyplot as plt
-matplotlib.use("TkAgg")
+
 
 """
 Static Example 0: Single vortex
@@ -18,12 +17,12 @@ if __name__ == "__main__":
     sq_array = SquareArray(N, N)
     dt = 0.05
     Nt = 1000
-    f = 0.1
-    Is = 0.9 * sq_array.horizontal_junctions()
-    T = 0.01
+    f = 0.2
+    Is = 0.5 * sq_array.horizontal_junctions()
+    T = 0.02
     problem = TimeEvolutionProblem(sq_array, time_step=dt,
                                    time_step_count=Nt, current_sources=Is,
                                    frustration=f, temperature=T)
     config = problem.compute()
-    config.animate(junction_quantity="theta", arrow_scale=None)
+    config.animate(junction_quantity="supercurrent")
     plt.show()
